@@ -1,7 +1,18 @@
-from typing import Dict, List, Optional
+from typing import Any, Dict, List, Optional, TypedDict
 
 from .base import Base
 
+
+class Operands(TypedDict):
+    filter_name: str
+    filter_value: Any
+
+
+class SearchQuery(TypedDict):
+    operator: str
+    operands: List[Operands]
+
+
 class Users(Base):
     @property
     def user_url(self):
@@ -22,9 +33,9 @@ class Users(Base):
         last_name: str = None,
         middle_name: str = None,
         create_user_as_pending: Optional[bool] = False,
-        attributes: Dict[str, str] = {},
+        attributes: Dict[str, str] = None,
     ):
-        data = {
+        data: Dict[str, Any] = {
             "email": email,
             "phone_number": phone_number,
             "name": {
@@ -42,9 +53,7 @@ class Users(Base):
         return self._get("{0}/{1}".format(self.user_url, user_id))
 
     def get_pending(
-        self,
-        limit: Optional[int] = None,
-        starting_after_id: Optional[str] = None
+        self, limit: Optional[int] = None, starting_after_id: Optional[str] = None
     ):
         query_params = {}
         if limit:
@@ -54,6 +63,36 @@ class Users(Base):
 
         return self._get("{0}/{1}".format(self.user_url, "pending"), query_params)
 
+    def search(
+        self,
+        limit: Optional[int] = None,
+        cursor: Optional[str] = None,
+        query: Optional[SearchQuery] = None,
+    ):
+        data: Dict[str, Any] = {}
+
+        if limit is not None:
+            data["limit"] = limit
+        if cursor is not None:
+            data["cursor"] = cursor
+        if query is not None:
+            data["query"] = query
+
+        return self._post("{0}/{1}".format(self.user_url, "search"), data)
+
+    def search_all(
+        self,
+        limit: Optional[int] = None,
+        cursor: Optional[str] = None,
+        query: Optional[SearchQuery] = None,
+    ):
+        while True:
+            results = self.search(limit, cursor, query)
+            yield results
+            cursor = results.json()["results_metadata"]["next_cursor"]
+            if cursor is None:
+                break
+
     def delete(self, user_id: str):
         return self._delete("{0}/{1}".format(self.user_url, user_id))
 
@@ -62,12 +101,13 @@ class Users(Base):
         user_id: str,
         emails: Optional[List[str]] = None,
         phone_numbers: Optional[List[str]] = None,
+        crypto_wallets: Optional[List[Dict[str, str]]] = None,
         first_name: Optional[str] = None,
         middle_name: Optional[str] = None,
         last_name: Optional[str] = None,
         attributes: Optional[Dict[str, str]] = {},
     ):
-        data = {}
+        data: Dict[str, Any] = {}
         name = {}
         if first_name:
             name.update({"first_name": first_name})
@@ -77,6 +117,8 @@ class Users(Base):
             name.update({"last_name": last_name})
         if name:
             data.update({"name": name})
+        if crypto_wallets:
+            data.update({"crypto_wallets": crypto_wallets})
 
         if emails:
             ems = []
@@ -100,3 +142,18 @@ class Users(Base):
 
     def delete_phone_number(self, phone_id: str):
         return self._delete("{0}/phone_numbers/{1}".format(self.user_url, phone_id))
+
+    def delete_webauthn_registration(self, webauthn_registration: str):
+        return self._delete(
+            "{0}/webauthn_registrations/{1}".format(
+                self.user_url, webauthn_registration
+            )
+        )
+
+    def delete_totp(self, totp_id: str):
+        return self._delete("{0}/totps/{1}".format(self.user_url, totp_id))
+
+    def delete_crypto_wallet(self, crypto_wallet_id: str):
+        return self._delete(
+            "{0}/crypto_wallets/{1}".format(self.user_url, crypto_wallet_id)
+        )
